# SPDX-License-Identifier: Apache-2.0

from .base_iface import NisporBaseIface
from .bond import NisporBond
from .bond import NisporBondSubordinate
from .bridge import NisporBridge
from .bridge import NisporBridgePort
from .clib_wrapper import NisporError
from .clib_wrapper import retrieve_net_state_json
from .iface import NisporIfaceState
from .mptcp import NisporMptcpState
from .route import NisporMultipathRoute
from .route import NisporRoute
from .route import NisporRouteState
from .route_rule import NisporRouteRule
from .route_rule import NisporRouteRuleState
from .state import NisporNetState
from .tun import NisporTun
from .veth import NisporVeth
from .vlan import NisporVlan
from .vxlan import NisporVxlan

__all__ = []



# SPDX-License-Identifier: Apache-2.0


class NisporIPAddr:
    def __init__(self, info):
        self._info = info

    @property
    def address(self):
        return self._info["address"]

    @property
    def prefix_len(self):
        return self._info["prefix_len"]

    @property
    def valid_lft(self):
        return self._info["valid_lft"]

    @property
    def preferred_lft(self):
        return self._info["preferred_lft"]


class NisporIPv4Addr(NisporIPAddr):
    @property
    def peer(self):
        return self._info.get("peer")


class NisporIPv6Addr(NisporIPAddr):
    pass


class NisporIPv4:
    def __init__(self, info):
        self._info = info
        self._address = []
        for addr_info in info.get("addresses", []):
            self._address.append(NisporIPv4Addr(addr_info))

    @property
    def addresses(self):
        return self._address

    def __str__(self):
        return f"{self._info}"


class NisporIPv6:
    def __init__(self, info):
        self._info = info
        self._address = []
        for addr_info in info.get("addresses", []):
            self._address.append(NisporIPv6Addr(addr_info))

    @property
    def addresses(self):
        return self._address

    def __str__(self):
        return f"{self._info}"



# SPDX-License-Identifier: Apache-2.0

from .ip import NisporIPv4
from .ip import NisporIPv6
from .sr_iov import NisporSriov
from .sr_iov import NisporSriovVf
from .ethtool import NisporEthtool


class NisporBaseIface:
    def __init__(self, info):
        self._info = info
        self._sub_state = None
        self._sr_iov = None
        self._ethtool = None
        if "sriov" in self._info:
            self._sr_iov = NisporSriov(self._info["sriov"])
        if "ethtool" in self._info:
            self._ethtool = NisporEthtool(self._info["ethtool"])
        if "sriov_vf" in self._info:
            self._sr_iov_vf = NisporSriovVf(self._info["sriov_vf"])

    def __str__(self):
        return f"{self._info}"

    @property
    def type(self):
        return self._info["iface_type"]

    @property
    def name(self):
        return self._info["name"]

    @property
    def state(self):
        return self._info["state"]

    @property
    def mtu(self):
        return self._info["mtu"]

    @property
    def flags(self):
        return self._info["flags"]

    @property
    def ipv4(self):
        if "ipv4" in self._info:
            return NisporIPv4(self._info["ipv4"])
        else:
            return None

    @property
    def ipv6(self):
        if "ipv6" in self._info:
            return NisporIPv6(self._info["ipv6"])
        else:
            return None

    @property
    def mac_address(self):
        return self._info.get("mac_address")

    @property
    def permanent_mac_address(self):
        return self._info.get("permanent_mac_address")

    @property
    def controller(self):
        return self._info.get("controller")

    @property
    def controller_type(self):
        return self._info.get("controller_type")

    @property
    def subordinate_state(self):
        return self._sub_state

    @subordinate_state.setter
    def subordinate_state(self, value):
        self._sub_state = value

    @property
    def sr_iov(self):
        return self._sr_iov

    @property
    def sr_iov_vf(self):
        return self._sr_iov_vf

    @property
    def ethtool(self):
        return self._ethtool

    @property
    def max_mtu(self):
        return self._info.get("max_mtu")

    @property
    def min_mtu(self):
        return self._info.get("min_mtu")


class NisporBaseSubordinateIface:
    def __init__(self):
        self._sub_info = None

    def __str__(self):
        return f"{self._sub_info}"



# SPDX-License-Identifier: Apache-2.0

from .base_iface import NisporBaseIface


class NisporIpoib(NisporBaseIface):
    def __init__(self, info):
        super().__init__(info)
        self._ipoib_info = self._info.get("ipoib", {})

    @property
    def pkey(self):
        return self._ipoib_info["pkey"]

    @property
    def mode(self):
        return self._ipoib_info["mode"]

    @property
    def umcast(self):
        return self._ipoib_info["umcast"]

    @property
    def base_iface(self):
        return self._ipoib_info.get("base_iface")



# SPDX-License-Identifier: Apache-2.0

from .base_iface import NisporBaseIface


class NisporMacVlan(NisporBaseIface):
    def __init__(self, info):
        super().__init__(info)
        self._mac_vlan_info = self._info.get("mac_vlan", {})

    @property
    def mode(self):
        return self._mac_vlan_info["mode"]

    @property
    def base_iface(self):
        return self._mac_vlan_info["base_iface"]

    @property
    def allowed_mac_addresses(self):
        return self._mac_vlan_info.get("allowed_mac_addresses")

    @property
    def mac_vlan_flags(self):
        return self._mac_vlan_info["flags"]



# SPDX-License-Identifier: Apache-2.0

from .base_iface import NisporBaseIface
from .base_iface import NisporBaseSubordinateIface


class NisporVRF(NisporBaseIface):
    def __init__(self, info):
        super().__init__(info)
        self._vrf_info = self._info.get("vrf", {})

    @property
    def subordinates(self):
        return self._vrf_info.get("subordinates", [])

    @property
    def table_id(self):
        return self._vrf_info.get("table_id")



# SPDX-License-Identifier: Apache-2.0

import json

from .clib_wrapper import retrieve_net_state_json
from .iface import NisporIfaceState
from .mptcp import NisporMptcpState
from .route import NisporRouteState
from .route_rule import NisporRouteRuleState


class NisporNetState:
    def __str__(self):
        return f"{self._info}"

    def __init__(self, info):
        self._info = info
        self._ifaces = NisporIfaceState(info.get("ifaces"))
        self._routes = NisporRouteState(info.get("routes"))
        self._route_rules = NisporRouteRuleState(info.get("rules"))
        if info.get("mptcp"):
            self._mptcp = NisporMptcpState(info["mptcp"])
        else:
            self._mptcp = None

    @property
    def ifaces(self):
        return self._ifaces

    @property
    def routes(self):
        return self._routes

    @property
    def route_rules(self):
        return self._route_rules

    @property
    def mptcp(self):
        return self._mptcp

    @staticmethod
    def retrieve():
        return NisporNetState(json.loads(retrieve_net_state_json()))



# SPDX-License-Identifier: Apache-2.0

from .base_iface import NisporBaseIface


class NisporTun(NisporBaseIface):
    def __init__(self, info):
        super().__init__(info)
        self._tun_info = self._info.get("tun", {})

    @property
    def mode(self):
        return self._tun_info["mode"]

    @property
    def owner(self):
        return self._tun_info.get("owner")

    @property
    def group(self):
        return self._tun_info.get("group")

    @property
    def pi(self):
        return self._tun_info["pi"]

    @property
    def vnet_hdr(self):
        return self._tun_info["vnet_hdr"]

    @property
    def multi_queue(self):
        return self._tun_info["multi_queue"]

    @property
    def persist(self):
        return self._tun_info["persist"]

    @property
    def num_queues(self):
        return self._tun_info.get("num_queues")

    @property
    def num_disabled_queues(self):
        return self._tun_info.get("num_disabled_queues")



# SPDX-License-Identifier: Apache-2.0


class NisporMptcpState:
    def __init__(self, info):
        self._addrs = [
            NisporMptcpAddress(addr_info) for addr_info in info.get("addresses", [])
        ]

    @property
    def enabled(self):
        return self._info["enabled"]

    @property
    def add_addr_accepted_limit(self):
        return self._info.get("add_addr_accepted_limit")

    @property
    def subflows_limit(self):
        return self._info.get("subflows_limit")

    @property
    def addresses(self):
        return self._addrs


class NisporMptcpAddress:
    def __init__(self, info):
        self._info = info

    @property
    def address(self):
        return self._info["address"]

    @property
    def id(self):
        return self._info.get("id")

    @property
    def port(self):
        return self._info.get("port")

    @property
    def flags(self):
        return self._info.get("flags")

    @property
    def iface(self):
        return self._info.get("iface")



# SPDX-License-Identifier: Apache-2.0

from .base_iface import NisporBaseIface


class NisporVeth(NisporBaseIface):
    def __init__(self, info):
        super().__init__(info)
        self._peer = self._info.get("veth", {}).get("peer")

    @property
    def peer(self):
        return self._peer



# SPDX-License-Identifier: Apache-2.0

from .base_iface import NisporBaseIface
from .bond import NisporBond
from .bond import NisporBondSubordinate
from .bridge import NisporBridge
from .bridge import NisporBridgePort
from .ipoib import NisporIpoib
from .mac_vlan import NisporMacVlan
from .mac_vtap import NisporMacVtap
from .tun import NisporTun
from .veth import NisporVeth
from .vlan import NisporVlan
from .vrf import NisporVRF
from .vxlan import NisporVxlan


class NisporIfaceState:
    def __init__(self, info):
        self._ifaces = {}
        if info:
            for iface_name, iface_info in info.items():
                self._ifaces[iface_name] = _iface_info_to_obj(iface_info)

    def __str__(self):
        return f"{self._ifaces}"

    def get(self, iface_name):
        return self._ifaces.get(iface_name)

    def __getitem__(self, iface_name):
        return self._ifaces[iface_name]

    def keys(self):
        for iface in self._ifaces.keys():
            yield iface

    def values(self):
        for iface in self._ifaces.values():
            yield iface


def _iface_info_to_obj(iface_info):
    iface_type = iface_info["iface_type"]
    ctrl_type = iface_info.get("controller_type")
    if iface_type == "bond":
        iface = NisporBond(iface_info)
    elif iface_type == "bridge":
        iface = NisporBridge(iface_info)
    elif iface_type == "tun":
        iface = NisporTun(iface_info)
    elif iface_type == "vlan":
        iface = NisporVlan(iface_info)
    elif iface_type == "vxlan":
        iface = NisporVxlan(iface_info)
    elif iface_type == "veth":
        iface = NisporVeth(iface_info)
    elif iface_type == "vrf":
        iface = NisporVRF(iface_info)
    elif iface_type == "mac_vlan":
        iface = NisporMacVlan(iface_info)
    elif iface_type == "mac_vtap":
        iface = NisporMacVtap(iface_info)
    elif iface_type == "Ipoib":
        iface = NisporIpoib(iface_info)
    else:
        iface = NisporBaseIface(iface_info)
    if ctrl_type == "bond":
        iface.subordinate_state = NisporBondSubordinate(iface_info)
    elif ctrl_type == "bridge":
        iface.subordinate_state = NisporBridgePort(iface_info)
    return iface
